@@ -10,7 +10,7 @@ import logging
 from enum import Enum
 
 from graph_notebook.network.EventfulNetwork import EventfulNetwork
-from gremlin_python.process.traversal import T
+from gremlin_python.process.traversal import T, Direction
 from gremlin_python.structure.graph import Path, Vertex, Edge
 from networkx import MultiDiGraph
 
@@ -81,9 +81,13 @@ def get_id(element):
         return str(element)
 
 
-def strip_and_truncate_label(old_label: str, max_len: int):
-    label = str(old_label).strip("[]'")
-    return label if len(label) <= max_len else label[:max_len - 3] + '...'
+def strip_and_truncate_label_and_title(old_label: str, max_len: int):
+    title = str(old_label).strip("[]'")
+    if len(title) <= max_len:
+        label = title
+    else:
+        label = title[:max_len - 3] + '...'
+    return title, label
 
 
 class GremlinNetwork(EventfulNetwork):
@@ -92,13 +96,11 @@ class GremlinNetwork(EventfulNetwork):
 
     1. A query which returns in the format of a path whose objects are in the order Vertex -> Edge -> Vertex.
     2. A query which returns a path as a valueMap. In this case, we will assume that the order is Vertex -> Edge -> Vertex.
-
-    In the future, we will work to support other types of paths and/or results. For now, they are all out of scope.
-    You can find more details on this in our design doc for visualization here: https://quip-amazon.com/R1jbA8eECdDd
     """
 
     def __init__(self, graph: MultiDiGraph = None, callbacks=None, label_max_length=DEFAULT_LABEL_MAX_LENGTH,
-                 group_by_property=T_LABEL, display_property=T_LABEL, ignore_groups=False):
+                 group_by_property=T_LABEL, display_property=T_LABEL, edge_display_property=T_LABEL,
+                 ignore_groups=False):
         if graph is None:
             graph = MultiDiGraph()
         if label_max_length < 3:
@@ -113,6 +115,10 @@ class GremlinNetwork(EventfulNetwork):
             self.display_property = json.loads(display_property)
         except ValueError:
             self.display_property = display_property
+        try:
+            self.edge_display_property = json.loads(edge_display_property)
+        except ValueError:
+            self.edge_display_property = edge_display_property
         self.ignore_groups = ignore_groups
         super().__init__(graph, callbacks)
 
@@ -237,46 +243,51 @@ class GremlinNetwork(EventfulNetwork):
             raise ValueError("results must be a list of paths")
 
         for path in results:
-            if not isinstance(path, Path):
-                raise ValueError("all entries in results must be paths")
+            if isinstance(path, Path):
+                if type(path[0]) is Edge or type(path[-1]) is Edge:
+                    raise INVALID_PATH_ERROR
 
-            if type(path[0]) is Edge or type(path[-1]) is Edge:
-                raise INVALID_PATH_ERROR
-
-            for i in range(len(path)):
-                if i == 0:
-                    self.add_vertex(path[i])
-                    continue
-
-                if type(path[i]) is Edge:
-                    edge = path[i]
-                    path_left = get_id(path[i - 1])
-                    path_right = get_id(path[i + 1])
-
-                    # If the edge is an object type but its vertices aren't, then the ID contained
-                    # in the edge won't be the same as the ids used to store those two vertices.
-                    # For example, g.V().inE().outV().path().by(valueMap()).by().by(valueMap(true)
-                    # will yield a V, E, V pattern where the first vertex is a dict without T.id, the edge
-                    # will be an Edge object, and the second vertex will be a dict with T.id.
-                    if edge.outV.id == path_left or edge.inV.id == path_right:
-                        from_id = path_left
-                        to_id = path_right
-                        self.add_path_edge(path[i], from_id, to_id)
-                    elif edge.inV.id == path_left or edge.outV.id == path_right:
-                        from_id = path_right
-                        to_id = path_left
-                        self.add_path_edge(path[i], from_id, to_id)
+                for i in range(len(path)):
+                    if isinstance(path[i], dict):
+                        self.insert_elementmap(path[i])
                     else:
-                        from_id = path_left
-                        to_id = path_right
-                        self.add_blank_edge(from_id, to_id, edge.id, label=edge.label)
-                    continue
-                else:
-                    from_id = get_id(path[i - 1])
+                        if i == 0:
+                            self.add_vertex(path[i])
+                            continue
 
-                self.add_vertex(path[i])
-                if type(path[i - 1]) is not Edge:
-                    self.add_blank_edge(from_id, get_id(path[i]))
+                        if type(path[i]) is Edge:
+                            edge = path[i]
+                            path_left = get_id(path[i - 1])
+                            path_right = get_id(path[i + 1])
+
+                            # If the edge is an object type but its vertices aren't, then the ID contained
+                            # in the edge won't be the same as the ids used to store those two vertices.
+                            # For example, g.V().inE().outV().path().by(valueMap()).by().by(valueMap(true)
+                            # will yield a V, E, V pattern where the first vertex is a dict without T.id, the edge
+                            # will be an Edge object, and the second vertex will be a dict with T.id.
+                            if edge.outV.id == path_left or edge.inV.id == path_right:
+                                from_id = path_left
+                                to_id = path_right
+                                self.add_path_edge(path[i], from_id, to_id)
+                            elif edge.inV.id == path_left or edge.outV.id == path_right:
+                                from_id = path_right
+                                to_id = path_left
+                                self.add_path_edge(path[i], from_id, to_id)
+                            else:
+                                from_id = path_left
+                                to_id = path_right
+                                self.add_blank_edge(from_id, to_id, edge.id, label=edge.label)
+                            continue
+                        else:
+                            from_id = get_id(path[i - 1])
+
+                        self.add_vertex(path[i])
+                        if type(path[i - 1]) is not Edge:
+                            self.add_blank_edge(from_id, get_id(path[i]))
+            elif isinstance(path, dict):
+                self.insert_elementmap(path)
+            else:
+                raise ValueError("all entries in results must be paths or maps")
 
     def add_vertex(self, v):
         """
@@ -319,13 +330,16 @@ class GremlinNetwork(EventfulNetwork):
 
             if self.display_property in [T_ID, 'id']:
                 label = str(node_id)
+                title = str(node_id)
             elif isinstance(self.display_property, dict):
                 try:
                     if self.display_property[title] in [T_ID, 'id']:
                         label = str(node_id)
+                        title = str(node_id)
                 except KeyError:
                     pass
-            data = {'label': str(label).strip("[]'"), 'title': title, 'group': group, 'properties': {'id': node_id, 'label': title}}
+            data = {'label': str(label).strip("[]'"), 'title': title, 'group': group,
+                    'properties': {'id': node_id, 'label': title}}
         elif type(v) is dict:
             properties = {}
             title = ''
@@ -336,7 +350,7 @@ class GremlinNetwork(EventfulNetwork):
             # Since it is needed for checking for the vertex label's desired grouping behavior in group_by_property
             if T.label in v.keys():
                 title = str(v[T.label])
-                label = strip_and_truncate_label(title, self.label_max_length)
+                title_plc, label = strip_and_truncate_label_and_title(title, self.label_max_length)
             for k in v:
                 if str(k) == T_ID:
                     node_id = str(v[k])
@@ -352,11 +366,11 @@ class GremlinNetwork(EventfulNetwork):
                 if isinstance(self.display_property, dict):
                     try:
                         if str(k) == self.display_property[title]:
-                            label = strip_and_truncate_label(str(v[k]), self.label_max_length)
+                            title, label = strip_and_truncate_label_and_title(str(v[k]), self.label_max_length)
                     except KeyError:
                         continue
                 elif str(k) == self.display_property:
-                    label = strip_and_truncate_label(str(v[k]), self.label_max_length)
+                    title, label = strip_and_truncate_label_and_title(str(v[k]), self.label_max_length)
 
             # handle when there is no id in a node. In this case, we will generate one which
             # is consistently regenerated so that duplicate dicts will be reduced to the same vertex.
@@ -390,18 +404,42 @@ class GremlinNetwork(EventfulNetwork):
             from_id = from_id if from_id != '' else edge.outV.id
             to_id = to_id if to_id != '' else edge.inV.id
             data['properties'] = {'id': edge.id, 'label': edge.label, 'outV': str(edge.outV), 'inV': str(edge.inV)}
-            self.add_edge(from_id, to_id, edge.id, edge.label, data)
+            if isinstance(self.edge_display_property, dict):
+                try:
+                    display_label = data['properties'][self.edge_display_property[edge.label]]
+                except KeyError:
+                    display_label = edge.label
+            else:
+                if self.edge_display_property == T_LABEL:
+                    display_label = edge.label
+                else:
+                    try:
+                        display_label = data['properties'][self.edge_display_property]
+                    except KeyError:
+                        display_label = edge.label
+            self.add_edge(from_id, to_id, edge.id, display_label, data)
         elif type(edge) is dict:
             properties = {}
             edge_id = ''
             edge_label = ''
+            if T.label in edge.keys():
+                edge_label = str(edge[T.label])
             for k in edge:
-                if str(k) == T_LABEL:
-                    edge_label = str(edge[k])
-                elif str(k) == T_ID:
+                if str(k) == T_ID:
                     edge_id = str(edge[k])
-
-                properties[k] = edge[k]
+                if type(edge[k]) is dict:  # Handle Direction properties, where the value is a map
+                    properties[k] = get_id(edge[k])
+                else:
+                    properties[k] = edge[k]
+                if self.edge_display_property is not T_LABEL:
+                    if isinstance(self.edge_display_property, dict):
+                        try:
+                            if str(k) == self.edge_display_property[edge_label]:
+                                edge_label = str(edge[k])
+                        except KeyError:
+                            continue
+                    elif str(k) == self.edge_display_property:
+                        edge_label = str(edge[k])
             data['properties'] = properties
             self.add_edge(from_id, to_id, edge_id, edge_label, data)
         else:
@@ -422,3 +460,28 @@ class GremlinNetwork(EventfulNetwork):
             edge_id = str(uuid.uuid4())
         edge_data = UNDIRECTED_EDGE if undirected else {}
         self.add_edge(from_id, to_id, edge_id, label, edge_data)
+
+    def insert_elementmap(self, e_map):
+        """
+        Add a vertex or edge that has been returned by an elementMap query step.
+
+        Any elementMap representations of edges must be directed, and contain both of the Direction.IN and Direction.OUT
+        properties. If the elementMap contains neither of these, then we assume it is a vertex.
+
+        :param e_map: A dictionary containing the elementMap representation of a vertex or an edge
+        """
+        # Handle directed edge elementMap
+        if Direction.IN in e_map.keys() and Direction.OUT in e_map.keys():
+            from_id = get_id(e_map[Direction.OUT])
+            to_id = get_id(e_map[Direction.IN])
+            # Ensure that the default nodes includes with edge elementMaps do not overwrite nodes
+            # with the same ID that have been explicitly inserted.
+            if not self.graph.has_node(from_id):
+                self.add_vertex(e_map[Direction.OUT])
+            if not self.graph.has_node(to_id):
+                self.add_vertex(e_map[Direction.IN])
+            self.add_path_edge(e_map, from_id, to_id)
+        # Handle vertex elementMap
+        else:
+            # Overwrite the the default node created by edge elementMap, if it exists already.
+            self.add_vertex(e_map)
